'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Created at: 2023-04-19
Purpose: Organize the constants
-----------------------------------------------------
'''
# SOFFOS_SERVICE_URL = "https://api.soffos.ai/service/"
# SOFFOS_SERVICE_URL = "http://localhost:8000/service/"
SOFFOS_SERVICE_URL = "https://dev-api.soffos.ai/service/"

SERVICES_LIST = [
    'answer-scoring', 
    'audio_converter',
    'chatbot/create',
    'chatbot',
    'chatbot/get',
    'chatbot/delete',
    'chatbot/sessions/get',
    'chatbot/sessions/delete',
    'discuss/create',
    'discuss', 
    'discuss/count',
    'discuss/delete',
    'documents/count',
    'documents/ingest', 
    'documents/delete',
    'documents/search',
    'email-analysis', 
    'emotion-detection', 
    'file-converter', 
    'flashcard-generation', 
    'intent-classification/confirmation', 
    'intent-classification', 
    'language-detection', 
    'logical-error-detection',
    'microlesson', 
    'natural-sql-generation',
    'natural-sql-generation/table/delete',
    'natural-sql-generation/table/get',
    'natural-sql-generation/table/ingest',
    'ner', 
    'paraphrase', 
    'profanity',
    'qna-generation',
    'question-answering',
    'review-tagger',
    'search-recommendations',
    'sentiment-analysis',
    'simplify',
    'string-similarity',
    'summarization',
    'table-generator',
    'tag',
    'transcript-correction',
    'translation',
    'batch-service/'
]

class ServiceString:
    '''
    Contains the list of Soffos services as attributes
    '''
    ANSWER_SCORING = 'answer-scoring'
    AUDIO_CONVERTER = 'audio-converter'
    CHAT_BOT_CREATE = 'chatbot/create'
    CHAT_BOT = 'chatbot'
    CHAT_BOTS_GET = 'chatbot/get'
    CHAT_BOTS_DELETE = 'chatbot/delete'
    CHAT_BOT_GET_USER_SESSIONS = 'chatbot/sessions/get'
    CHAT_BOT_DELETE_USER_SESSIONS = 'chatbot/sessions/delete'
    DISCUSS_CREATE = 'discuss/create'
    DISCUSS_QUERY = 'discuss'
    DISCUSS_RETRIEVE = 'discuss/count'
    DISCUSS_DELETE = 'discuss/delete'
    DISCUSS_COUNT = 'discuss/count'
    DOCUMENTS_COUNT = 'documents/count'
    DOCUMENTS_INGEST = 'documents/ingest'
    DOCUMENTS_DELETE = 'documents/delete'
    DOCUMENTS_SEARCH = 'documents/search'
    EMAIL_ANALYSIS = 'email-analysis'
    EMOTION_DETECTION = 'emotion-detection'
    FILE_CONVERTER = 'file-converter'
    FLASHCARD_GENERATION = 'flashcard-generation'
    INTENT_CLASSIFICATION = 'intent-classification'
    LANGUAGE_DETECTION = 'language-detection'
    LOGICAL_ERROR_DETECTION = 'logical-error-detection'
    MICROLESSON = 'microlesson'
    NATURAL_S_Q_L_GENERATION = 'natural-sql-generation'
    N_E_R = 'ner'
    PARAPHRASE = 'paraphrase'
    PROFANITY = 'profanity'
    QN_A_GENERATION = 'qna-generation'
    QUESTION_ANSWERING = "question-answering"
    REVIEW_TAGGER = 'review-tagger'
    SEARCH_RECOMMENDATIONS = 'search-recommendations'
    SENTIMENT_ANALYSIS = 'sentiment-analysis'
    SIMPLIFY = 'simplify'
    STRING_SIMILARITY = 'string-similarity'
    SUMMARIZATION = 'summarization'
    TABLE_DELETE = 'natural-sql-generation/table/delete'
    TABLE_GET = 'natural-sql-generation/table/get'
    TABLE_GENERATOR = 'table-generator'
    TABLE_INGEST = 'natural-sql-generation/table/ingest/'
    TAG = 'tag'
    TRANSCRIPT_CORRECTION = 'transcript-correction'
    TRANSLATION = 'translation'
    BATCH_SERVICE = 'batch-service2'


FORM_DATA_REQUIRED = [ServiceString.FILE_CONVERTER, ServiceString.AUDIO_CONVERTER]



'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Updated at: 2023-10-09
Purpose: Input/Output description for Sentiment Analysis Service
-----------------------------------------------------
'''
from .service_io import ServiceIO
from ..constants import ServiceString


class SentimentAnalysisIO(ServiceIO):
    service = ServiceString.SENTIMENT_ANALYSIS
    required_input_fields = ["text"]
    optional_input_fields = ["sentence_split","sentence_overlap"]
    input_structure = {
        "text": str, 
        "sentence_split": int, 
        "sentence_overlap": bool
    }

    output_structure = {
        "sentiment_breakdown": dict,
        "sentiment_overall": dict,
        "sentiment": str
    }


    @classmethod
    def special_validation(self, payload):
        
        if payload["sentence_split"] == 1 and payload["sentence_overlap"] == True:
            return False, 'Value "sentence_overlap" must be false when "sentence_split" is set to 1.'

        return super().special_validation(payload)


'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Updated at: 2023-10-09
Purpose: Input/Output description for Documents Delete Service
-----------------------------------------------------
'''
from .service_io import ServiceIO
from ..constants import ServiceString


class DocumentsDeleteIO(ServiceIO):
    service = ServiceString.DOCUMENTS_DELETE
    required_input_fields = ["document_ids"]
    optional_input_fields = []
    input_structure = {
        "document_ids": list
    }

    output_structure = {
        "success": bool
    }




'''
Copyright (c)2022 - Soffos.ai - All rights reserved
Updated at: 2023-10-09
Purpose: Easily use Documents Search Service
-----------------------------------------------------
'''
from .service import SoffosAIService
from .input_config import InputConfig
from soffosai.common.constants import ServiceString
from typing import Union



class DocumentsSearchService(SoffosAIService):
    '''
    General HTTP service client.
    '''

    def __init__(self, **kwargs) -> None:
        service = ServiceString.DOCUMENTS_SEARCH
        super().__init__(service, **kwargs)
    
    def __call__(self, user:str, query:str=None, document_ids:list=None, top_n_keyword:int=5, top_n_natural_language:int=5, filters:dict=None, date_from:str=None, date_until:str=None) -> dict:
        '''
        Call the Documents Search Service
        
        :param user: The ID of the user accessing the Soffos API.
            Soffos assumes that the owner of the api is an application (app) and that app has users.
            Soffos API will accept any string."
        
        :param query: The text to be used to match passages from ingested documents.
        :param document_ids: Passing document IDs will confine the search to those documents.
        :param top_n_keyword: The number of document passages to be retrieved using keyword search.
        :param top_n_natural_language: The number of document passages to be retrieved using Machine
            Learning-based semantic search.
        :param filters: The filters field can be used to narrow down the search to only the
            documents meeting certain metadata-based criteria, or even returning all
            the filtered documents when query is left null.
        :param date_from: Filters passages to those ingested at or after the specified ISO-8601
            formatted date.
        :param date_until: Filters passages to those ingested before the specified ISO-8601 formatted
            date.
        :return: items: None
        :Examples
        Detailed examples can be found at `Soffos Github Repository <https://github.com/Soffos-Inc/soffosai-python/tree/master/samples/services/documents.py>`_
        '''
        return super().__call__(user=user, query=query, document_ids=document_ids, top_n_keyword=top_n_keyword, top_n_natural_language=top_n_natural_language, filters=filters, date_from=date_from, date_until=date_until)

    def set_input_configs(self, name:str, query:Union[str, InputConfig]=None, document_ids:Union[list, InputConfig]=None, top_n_keyword:Union[int, InputConfig]=5, top_n_natural_language:Union[int, InputConfig]=5, filters:Union[dict, InputConfig]=None, date_from:Union[str, InputConfig]=None, date_until:Union[str, InputConfig]=None):
        super().set_input_configs(name=name, query=query, document_ids=document_ids, top_n_keyword=top_n_keyword, top_n_natural_language=top_n_natural_language, filters=filters, date_from=date_from, date_until=date_until)

    @classmethod
    def call(self, user:str, query:str=None, document_ids:list=None, top_n_keyword:int=5, top_n_natural_language:int=5, filters:dict=None, date_from:str=None, date_until:str=None) -> dict:
        '''
        Call the Documents Search Service
        
        :param user: The ID of the user accessing the Soffos API.
            Soffos assumes that the owner of the api is an application (app) and that app has users.
            Soffos API will accept any string."
        
        :param query: The text to be used to match passages from ingested documents.
        :param document_ids: Passing document IDs will confine the search to those documents.
        :param top_n_keyword: The number of document passages to be retrieved using keyword search.
        :param top_n_natural_language: The number of document passages to be retrieved using Machine
            Learning-based semantic search.
        :param filters: The filters field can be used to narrow down the search to only the
            documents meeting certain metadata-based criteria, or even returning all
            the filtered documents when query is left null.
        :param date_from: Filters passages to those ingested at or after the specified ISO-8601
            formatted date.
        :param date_until: Filters passages to those ingested before the specified ISO-8601 formatted
            date.
        :return: items: None
        :Examples
        Detailed examples can be found at `Soffos Github Repository <https://github.com/Soffos-Inc/soffosai-python/tree/master/samples/services/documents.py>`_
        '''
        return super().call(user=user, query=query, document_ids=document_ids, top_n_keyword=top_n_keyword, top_n_natural_language=top_n_natural_language, filters=filters, date_from=date_from, date_until=date_until)

